/**********************************************************************************
* C Library for the display modules
* XV-GDM128X64 and XV-GDM128X64-RGB from XV-Display.de
*
* Section: Config.h
*
* Revision 1
* 2013 Feb. 07
*
* AVR and AVR Xmega microcontrollers,
* SoftwareSPI and HardwareSPI supported.
*
*
* Recommended development environment:
*
* Atmel Studio 6
* ---> http://www.atmel.com/Microsite/atmel_studio6/default.aspx
* Developmentboard Nano from dev-tools.de.
* ---> http://www.dev-tools.de/index.php/development-boards/developmentboard-nano
*
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
*
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
*
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* the Dev-Tools Nano development board.
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
*
*
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file.
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/


    /***
    Target architecture
    Currently available architectures: ARCH_AVR8, ARCH_XMEGA 
    ***/

    #define ARCH  ARCH_AVR8
    //  #define ARCH  ARCH_XMEGA

	

    /***
    Interface mode HardwareSPI/SoftwareSPI
    If DISPLAY_SPI_MODE_HARDWARESPI is true, HardwareSPI is used, else SoftwareSPI is used
    ***/

    #define DISPLAY_SPI_MODE_HARDWARESPI false  // true or false


    /***
	Backlight control
	If DISPLAY_CONTROL_BACKLIGHT is true, backlight signal pins will be controlled by library.
	***/
	
    #define DISPLAY_BACKLIGHT_CONTROL	true // true or false

    #define DISPLAY_BACKLIGHT_INVERTED	false // true or false


	
    /***
	Embedded fonts
	Font 2: width:10px, height 16px, memory usage about 5326 Byte
	***/

    #define EMBED_FONT2					true // true or false



    #if (ARCH == ARCH_AVR8)

    /*********************************************************************************************
    AVR
	Configuration of hardware related section for AVR microcontroller
    *********************************************************************************************/    

        /***
        Display SPI Signal assignment
        ***/

        #define DISPLAY_SPI_PORT         PORTB
        #define DISPLAY_SPI_DATADIR      DDRB
        #define DISPLAY_SPI_INPORT       PINB
        #define DISPLAY_SPI_MOSI         PB5  // DATA
        #define DISPLAY_SPI_SCK          PB7  // CLK

		
        #define DISPLAY_SPI_SPEED_FCPU_DIV 2 // allowed values: 2, 4, 8, 16, 32, 64, 128 (for spi module only) 


        /***
        Display Control Signal assignment   
        ***/

        #define DISPLAY_CONTROL_PORT     PORTB
        #define DISPLAY_CONTROL_DATADIR  DDRB
        #define DISPLAY_CONTROL_INPORT   PINB
        #define DISPLAY_SIGNAL_CS        PB4
		#define DISPLAY_SIGNAL_A0        PB3
		#define DISPLAY_SIGNAL_RESET     PB2	
		
		
        /***
        Backlight Control Signals (required if Backlight is controlled by lib, DISPLAY_BACKLIGHT_CONTROL = true) 
        ***/		
		#define DISPLAY_BACKLIGHT_PORT		PORTA
		#define DISPLAY_BACKLIGHT_DATADIR   DDRA
		
		#define DISPLAY_BACKLIGHT_RED		PA0  // Pin assignment for RED LED
		#define DISPLAY_BACKLIGHT_GREEN		PA1  // Pin assignment for GREEN LED
		#define DISPLAY_BACKLIGHT_BLUE		PA2  // Pin assignment for BLUE LED
		
		
    #elif (ARCH == ARCH_XMEGA)
    
    /*********************************************************************************************
    XMEGA
	Configuration of hardware related section for XMEGA microcontroller
	*********************************************************************************************/    
    
        /***
        Display SPI Signal assignment
        ***/

        #define DISPLAY_SPI_PORT         PORTD
        #define DISPLAY_SPI_MODULE       SPID  // Hardware SPI module
        #define DISPLAY_SPI_MOSI         PIN5  // DATA
        #define DISPLAY_SPI_SCK          PIN7  // CLK

        #define DISPLAY_SPI_SPEED_FCPU_DIV 2  // allowed values: 2, 4, 8, 16, 32, 64, 128 (for spi module only) 

        /***
        Display Control Signal assignment   
        ***/

        #define DISPLAY_CONTROL_PORT     PORTD

        #define DISPLAY_SIGNAL_CS        PIN4
    	#define DISPLAY_SIGNAL_A0        PIN3
    	#define DISPLAY_SIGNAL_RESET     PIN2
    
	
        /***
        Backlight Control Signals (required if Backlight is controlled by lib, DISPLAY_BACKLIGHT_CONTROL = true) 
        ***/		
		#define DISPLAY_BACKLIGHT_PORT		PORTA
		
		#define DISPLAY_BACKLIGHT_RED		PIN0  // Pin assignment for RED LED
		#define DISPLAY_BACKLIGHT_GREEN		PIN1  // Pin assignment for GREEN LED
		#define DISPLAY_BACKLIGHT_BLUE		PIN2  // Pin assignment for BLUE LED	
	
    #else
        #error The peripheral driver is not currently available for your selected architecture.
    #endif


  







