/**********************************************************************************
* C Library for the display modules
* XV-GDM128X64 and XV-GDM128X64-RGB from XV-Display.de
*
* Section: Display.h
*
* Revision 1
* 2013 Feb. 07
*
* AVR and AVR Xmega microcontrollers,
* SoftwareSPI and HardwareSPI supported.
*
*
* Recommended development environment:
*
* Atmel Studio 6
* ---> http://www.atmel.com/Microsite/atmel_studio6/default.aspx
* Developmentboard Nano from dev-tools.de.
* ---> http://www.dev-tools.de/index.php/development-boards/developmentboard-nano
*
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
*
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
*
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* the Dev-Tools Nano development board.
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
*
*
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file.
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/

#define ARCH_AVR8  0
#define ARCH_XMEGA 1

#ifndef __XVDisplay_h__
#define __XVDisplay_h__

#include "Config.h"

#ifndef true
#define true 1
#endif

#ifndef false
#define false 0
#endif






void Display_Clear(void);
void Display_Fill(void);
void Display_DrawPixel(int16_t x, int16_t y);
void Display_DrawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1);
void Display_DrawRect(int16_t x, int16_t y, int16_t w, int16_t h);
void Display_DrawRect_Fill(int16_t x, int16_t y, int16_t w, int16_t h);
void Display_DrawCircle(int16_t x0, int16_t y0, int16_t radius);
void Display_DrawImage(int16_t x1, int16_t y1, const uint8_t *image);
void _Display_Char(int16_t x, int16_t y, char c, uint8_t width, uint8_t height);
void _Display_DrawPixelChar(int16_t x, int16_t y, uint8_t mode);

void Display_Text(int16_t x, int16_t y, char *string);
void Display_Text_F(int16_t x, int16_t y, const char *string);
void SetTextMode(uint8_t mode);

void Display_Decimal8(int16_t x, int16_t y, uint8_t d);
void Display_Decimal16(int16_t x, int16_t y, uint16_t d);

#if DISPLAY_BACKLIGHT_CONTROL == true
  void Display_SetBacklight_By_Color(uint8_t color);
#endif

void Display_Init(void);
void Display_Update(void);
void Display_SetContrast(uint8_t i);

#define nop() __asm volatile ("nop")




#define DISPLAY_WIDTH 128  // Dimensions in landscape mode
#define DISPLAY_HEIGHT 64
uint8_t display_memory[DISPLAY_WIDTH][DISPLAY_HEIGHT>>3];  // 128x8 Byte = 1.024Byte sram usage

uint8_t PenMode;

#define pmSET 0    // Pen Mode: Set Pixel
#define pmCLEAR 1  // Pen Mode: Clear Pixel
#define pmEXOR 2   // Pen Mode: Exor Pixel
		
#define SetPenMode(mode); PenMode = mode;

uint8_t FontIndex;

#define SetFont(index); FontIndex = index;

#define FONT_1 1
#define FONT_2 2

uint8_t DisplayMode;
#define dmLandscape 0
#define dmPortrait  1

uint8_t TextMode;

#define tmNormal_bp		0
#define tmBrushSolid_bp 1

#define tmNormal		0
#define tmInverse		1
#define tmBrushSolid	2
#define tmBrushClear	3
#define tmInit			5





#define SetDisplayMode(mode);	DisplayMode = mode;

// RGB Backlight
#define blOFF      0
#define blRED      1
#define blGREEN    2
#define blYELLOW   3
#define blBLUE     4
#define blPURPLE   5
#define blCYAN     6
#define blWHITE    7


#endif