/**********************************************************************************
* C Library for the display modules
* XV-GDM128X64 and XV-GDM128X64-RGB from XV-Display.de
*
* Section: GraphicDisplayModule128x64.c
*
* Revision 1
* 2013 Feb. 07
*
* AVR and AVR Xmega microcontrollers,
* SoftwareSPI and HardwareSPI supported.
*
*
* Recommended development environment:
*
* Atmel Studio 6
* ---> http://www.atmel.com/Microsite/atmel_studio6/default.aspx
* Developmentboard Nano from dev-tools.de.
* ---> http://www.dev-tools.de/index.php/development-boards/developmentboard-nano
*
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
*
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
*
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* the Dev-Tools Nano development board.
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
*
*
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file.
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/


#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "Display.h"

extern const uint8_t image1[];

// Prototypes of helper routines for this demo
void DisplayContrastBargraph(uint8_t i);


int main(void)
{
	
	uint8_t i, x, y;
	
	_delay_ms(10);
	Display_Init();
	
	Display_Clear();
	Display_Update();
	Display_SetBacklight_By_Color(blWHITE);
	
    while(1)
    {
        
		
		Display_Clear();
		Display_DrawImage(0, 20, image1);
		Display_Update();
		Display_Clear();
		_delay_ms(8000);		
		
		
		
		Display_Clear();

        // Welcome
		Display_Text_F(12+12, 10, PSTR("Library for"));
		Display_Text_F(12+8, 20,  PSTR("XV-GDM128X64"));
		Display_Text_F(12, 30,    PSTR("Display module"));		
		Display_Update();
		Display_Clear();
		_delay_ms(8000);



		/*****************************************************************************************
        Display graphic
		
		void Display_DrawPixel(int16_t x, int16_t y);
		void Display_DrawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1);
		void Display_DrawRect(int16_t x, int16_t y, int16_t w, int16_t h);
		void Display_DrawRect_Fill(int16_t x, int16_t y, int16_t w, int16_t h);
		void Display_DrawCircle(int16_t x0, int16_t y0, int16_t radius);
		
		SetPenMode(mode)
		Parameter: mode = pmSET, pmCLEAR, pmEXOR
		pmSet	: set pixel
		pmCLEAR	: clear pixel
		pmEXOR	: exor pixel
		*****************************************************************************************/		
		
	
		Display_Text_F(0, 5, PSTR("Draw pixel"));		
		for (x=0; x<128; x+=10)
		  for (y=20; y<64; y+=5) Display_DrawPixel(x, y);
		Display_Update();
		Display_Clear();
		_delay_ms(2500);		
		
		Display_Text_F(0, 5, PSTR("Draw line"));
		Display_DrawLine(0, 15, 127, 63);
		Display_Update();
		Display_Clear();		
		_delay_ms(2500);
				
		Display_Text_F(0, 5, PSTR("Draw rect"));
		Display_DrawRect(10, 20, 88, 40);
		Display_DrawRect_Fill(20, 30, 88, 20);
		Display_Update();
		Display_Clear();
		_delay_ms(2500);		
				
		Display_Text_F(0, 5, PSTR("Draw circle"));
		Display_DrawCircle(30, 80, 50);
		Display_Update();
		Display_Clear();
		_delay_ms(2500);

		Display_Text_F(0, 5, PSTR("Pen mode:"));
		Display_Text_F(0, 20, PSTR("Set pixel"));		
		Display_Text_F(0, 30, PSTR("Clear pixel"));	
		Display_Text_F(0, 40, PSTR("Exor pixel"));						
		
		// SetPenMode(pmEXOR);
		// SetPenMode(pmCLEAR);
		// SetPenMode(pmSET);
		Display_Update();
		Display_Clear();
		_delay_ms(5000);
		
		Display_Text_F(0, 5, PSTR("Pen mode EXOR"));		
		Display_Update();
		//Display_Clear();
		_delay_ms(1000);
	
	    SetPenMode(pmEXOR);
		for (x=0; x<128; x+=10)
		  for (y=20; y<64; y+=5) Display_DrawPixel(x, y);	// Pixel
		Display_DrawLine(0, 15, 127, 63);
		Display_DrawRect(10, 20, 88, 40);
		Display_DrawRect_Fill(20, 30, 88, 20);		
		Display_DrawCircle(30, 80, 50);
		Display_Update();
		Display_Clear();
		SetPenMode(pmSET);
		_delay_ms(5000);		

		/*****************************************************************************************
        Display text
		
		void Display_Text(uint8_t x, uint8_t y, char *string);
		void Display_Text_F(uint8_t x, uint8_t y, const char *string);
		
		SetFont(index)
		Parameter: index = FONT_1 or FONT_2
		Font 1: width: 6px, height: 8px
		Font 2: width:10px, height 16px, memory usage about 5326 Byte
		(Font 2 available if EMBED_FONT2 = true, see Config.h)
		
		SetTextMode(mode)
		Parameter: mode = (tmNormal or tmInverse) and (tmBrushSolid or tmBrushClear)
		tmNormal	: display text normal
		tmInverse	: display text inverse
		tmBrushSolid: backround of text is solid (inversed to pen color)
		tmBrushClear: background of text is not visible
		tmInit		: normal and solid brush
	
		*****************************************************************************************/		
		
		
		Display_Clear();
		Display_Text_F(0, 5, PSTR("Example text!"));
		Display_Text_F(0, 25, PSTR("Font 1"));
		Display_Update();
		_delay_ms(2000);
		
		Display_Clear();
		SetFont(FONT_2);
		Display_Text_F(0, 5, PSTR("Example text!"));
		Display_Text_F(0, 25, PSTR("Font 2"));
		Display_Update();
		_delay_ms(2000);
		
		
		Display_Clear();
		SetFont(FONT_1);
		Display_Text_F(0, 0, PSTR("Text mode:"));
		Display_Text_F(0, 20, PSTR("normal+brushsolid"));
		//SetTextMode(tmBrushClear);		
		Display_Text_F(0, 30, PSTR("normal+brushclear"));		
		//SetTextMode(tmBrushSolid);		
		//SetTextMode(tmInverse);
		Display_Text_F(0, 40, PSTR("inverse+brushsolid"));
		//SetTextMode(tmBrushClear);
		Display_Text_F(0, 50, PSTR("inverse+brushclear"));	
		
		SetTextMode(tmNormal);
		SetTextMode(tmBrushSolid);		
		Display_Update();
		_delay_ms(5000);		
		
		
		
	
		Display_Clear();
		Display_Text_F(0, 0, PSTR("Landscape"));
		Display_Text_F(0, 10, PSTR("Mode"));		
		Display_Update();
		_delay_ms(2000);
				
		Display_Clear();				
		SetDisplayMode(dmPortrait);
		Display_Text_F(0, 0, PSTR("Portrait"));
		Display_Text_F(0, 10, PSTR("Mode"));			
		SetDisplayMode(dmLandscape);				
		Display_Update();
		_delay_ms(2000);				
			
			
							
		// Chars of font 1
		Display_Clear();
		Display_Text_F(0, 5, PSTR("Font 1 chars ..."));
		Display_Update();
		_delay_ms(2000);
		
		x = 0; y = 0; i = 0;
		char s[3];s[1] = 0;
		
		do {
						
			s[0] = i;
						
			if (x >= 128)
			{
				x = 0;
				y += 8;
			}
			
			Display_Text(x, y, s);
			
			x += 8;
			if (y >= 64)
			{				
				y = 0; x = 0;
				Display_Update();
				Display_Clear();
				_delay_ms(5000);
			}
					
			i++;
				
		} while( i );
		
		Display_Update();
		
		_delay_ms(5000);
		
		/*****************************************************************************************
		Decimal numbers
		
		void Display_Decimal8(int16_t x, int16_t y, uint8_t d);
		void Display_Decimal16(int16_t x, int16_t y, uint16_t d);
		
		******************************************************************************************/			
		
		Display_Clear();
		Display_Text_F(0, 0, PSTR("Display Decimal"));
		Display_Update();
		_delay_ms(1000);
		
		for (uint16_t d=0; d<2001; d++)
		{
			Display_Decimal8(40, 30, d);
			Display_Decimal16(40, 40, d<<4);
		
		    Display_Update();
			if (d < 20) _delay_ms(200);			
			
		}
		
		_delay_ms(2000);
		
		/*****************************************************************************************
		Backlight
		
		void Display_SetBacklight_By_Color(uint8_t color);
		Parameter color: blOFF, blRED, blGREEN, blYELLOW, blBLUE, blPURPLE, blCYAN, blWHITE
		
		Available if backlight controlled by lib (DISPLAY_BACKLIGHT_CONTROL = true), see Config.h.
		******************************************************************************************/	
	
		#if DISPLAY_BACKLIGHT_CONTROL == true
		
		Display_Clear();
		Display_Text_F(0, 5, PSTR("Backlight color ..."));
		
		Display_Text_F(0, 25, PSTR("WHITE"));
		Display_Update();
        Display_SetBacklight_By_Color(blWHITE);
		_delay_ms(2000);
		
		Display_Text_F(0, 25, PSTR("RED  "));
		Display_Update();
		Display_SetBacklight_By_Color(blRED);
		_delay_ms(2000);		
		
		Display_Text_F(0, 25, PSTR("GREEN"));
		Display_Update();
		Display_SetBacklight_By_Color(blGREEN);
		_delay_ms(2000);		
		
		Display_Text_F(0, 25, PSTR("BLUE "));
		Display_Update();
		Display_SetBacklight_By_Color(blBLUE);
		_delay_ms(2000);		
		
		Display_Text_F(0, 25, PSTR("YELLOW"));
		Display_Update();
		Display_SetBacklight_By_Color(blYELLOW);
		_delay_ms(2000);		
		
		Display_Text_F(0, 25, PSTR("PURPLE"));
		Display_Update();
		Display_SetBacklight_By_Color(blPURPLE);
		_delay_ms(2000);		
		
		Display_Text_F(0, 25, PSTR("CYAN  "));
		Display_Update();
		Display_SetBacklight_By_Color(blCYAN);
		_delay_ms(2000);		
		
		Display_SetBacklight_By_Color(blWHITE);
		Display_Text_F(0, 25, PSTR("Backlight OFF"));
		Display_Update();
		_delay_ms(1500);
		Display_SetBacklight_By_Color(blOFF);
		_delay_ms(2000);		
		Display_Text_F(0, 25, PSTR("Backlight ON "));		
		Display_SetBacklight_By_Color(blWHITE);
		Display_Update();		
		_delay_ms(1000);		
		#endif	
		
		
		
		/*****************************************************************************************
		Set contrast of display module
		
		void Display_SetContrast(uint8_t i);
		Parameter: i = 0..40, center is 20
		*****************************************************************************************/
		
		
		Display_Clear();
		Display_Text_F(5, 5, PSTR("Set Contrast ..."));
		
		DisplayContrastBargraph(41); // display bar graph box
		
		Display_Update();
		
		for (i=20; i<41; i++)	// contrast 20->40
		{
			Display_SetContrast(i);
			DisplayContrastBargraph(i);
			Display_Update();			
			_delay_ms(30);
		}
		for (i=40; i>1; i--)	// contrast 40->1
		{
			Display_SetContrast(i);			
			DisplayContrastBargraph(i);
			Display_Update();						
			_delay_ms(30);
		}
		for (i=0; i<21; i++)	// contrast 0->20
		{
			Display_SetContrast(i);			
			DisplayContrastBargraph(i);
			Display_Update();					
			_delay_ms(30);
		}		
		
		
		
    }
}


// Helper routines


/***
DisplayContrastBargraph
Parameter: i = 0..40 contrast, 41 display box
***/

void DisplayContrastBargraph(uint8_t i)
{
	
	if (i > 41) return;
	
	else if (41 == i)
		Display_DrawRect(19, 29, 83, 5);  // display box
	else
	{
		
		Display_DrawRect_Fill(20, 30, i<<1, 3);
		SetPenMode(pmCLEAR);
		Display_DrawRect_Fill(20+(i<<1), 30, 80-(i<<1), 3);
		SetPenMode(pmSET);		
				
	}
		
}

